<?php

namespace Drupal\syncart_order_status\Service;

use Drupal\Core\Entity\EntityTypeManager;

/**
 * Orders service.
 */
class Order {

  /**
   * Drupal\Core\Entity\EntityTypeManager definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  public function __construct(EntityTypeManager $entity_type_manager) {
    $this->orderStorage = $entity_type_manager->getStorage('commerce_order');
    $this->taxonomyTermStorage = $entity_type_manager->getStorage('taxonomy_term');

    $this->activeOrdersQuery = $this->orderStorage->getQuery()
      ->condition('state', 'completed')
      ->sort('field_status');
    $this->statusesQuery = $this->taxonomyTermStorage->getQuery()
      ->condition('status', 1)
      ->condition('vid', 'order_status');
  }

  public function setOrderStatus($order_id, $status_id) {
    $order = $this->orderStorage->load($order_id);
    $status = $this->taxonomyTermStorage->load($status_id);
    if ($order) {
      $order->set('field_status', $status_id);
      $order->save();
      return "Order status changed to {$status->name->value}";
    }
  }

  /**
   * Получить активные заказы.
   */
  public function getActiveOrders() {
    $entities = [];
    $ids = $this->activeOrdersQuery->execute();
    foreach ($ids as $id) {
      $entities[$id] = $this->orderStorage->load($id);
    }
    return $entities;
  }

  /**
   * Получить данные пользователя из заказа.
   */
  public function getBillingInformation(object $order) {
    $billing_profile = $order->billing_profile->entity;
    return [
      'name' => $billing_profile->field_customer_name->value ?? '',
      'surname' => $billing_profile->field_customer_surname->value ?? '',
      'phone' => $billing_profile->field_customer_phone->value ?? '',
      'email' => $billing_profile->field_customer_email->value ?? '',
    ];
  }

  /**
   * Получить все доступные статусы.
   */
  public function getStatuses() {
    $entities = [];
    $ids = $this->statusesQuery->execute();
    foreach ($ids as $id) {
      $entities[$id] = $this->taxonomyTermStorage->load($id);
    }
    return $entities;
  }
}
